import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Polygon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Random;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * @MemoryGUI.java     14.03.2011
 * build: 14.03.2011
 * Copyright 2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Memory
 * Memory is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Memory is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Memory;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */

/**
 * Spielfl&auml;che des Memory-Spiels Dient auch als Controller des Programms
 * 
 */
public class MemoryGUI extends JFrame {

	/**
	 * der letzte angeklickte Stein
	 */
	private MemoryStein lastStein;
	/**
	 * Seitenlaenge des Spiels[Anzahl Steine]
	 */
	private int gridCount = 6;
	/**
	 * Steingroesse[px]
	 */
	private int steinSize = 90;
	/**
	 * Array aller Steine[gridCount x gridCount]
	 */
	private MusterStein[] steine;
	/**
	 * die seitlichen Fl&auml;chen zur Verwaltung der Spielteilnehmer
	 */
	private UserPanel[] userPanels = { new UserPanel(true, 1),
			new UserPanel(false, 2) };
	/**
	 * Index des aktiven Spielers. Der Index entspricht demjenigen des
	 * UserPanel-Arrays
	 */
	private int activeUser = 0;

	/**
	 * Hilfsvariable um die Zeitmessung unabh&auml;ngig vom Umdrehen machen.
	 */
	private boolean start = false;

	/**
	 * Spielmodus kann offen oder klassisch sein
	 */
	int mode;

	// GUI-Elemente
	private JPanel gridPanel, buttonPanel;
	private JButton turnButt, closeButt, newButt, startButt;
	private JTextField countField;
	private JLabel restLabel;

	public MemoryGUI() {
		// Spielmodus
		mode = askForMode();
		if (mode == JOptionPane.CANCEL_OPTION)
			System.exit(0);
		// GUI erstellen
		initGUI();
		// Steine erstellen und setzen
		createMemorySteinArray();
		// Restanzeige
		showRest();
		if (mode == 0) {
			switchColors();
			enableMemory(false);
		}
		// JFrame-Grundeinstellungen
		this.setSize(new Dimension(gridCount * steinSize + 2
				* userPanels[0].getPreferredSize().width, gridCount * steinSize
				+ countField.getPreferredSize().height
				+ buttonPanel.getPreferredSize().height + 20));

		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setLocationRelativeTo(null);
		this.setTitle("Memory");
		this.setVisible(true);
	} // Konstruktor

	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = MemoryGUI.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		}
		System.err.println("Datei nicht gefunden: " + path);
		return null;
	}

	/**
	 * Stellt den Dialog zur Modus-Auswahl bereit
	 * 
	 * @return int, 0 = klassisch, 1 = offen
	 */
	private int askForMode() {
		ImageIcon ico = createImageIcon("img/Icon.png");
		String[] options = { "Klassisch", "Offen", "Abbrechen" };
		return JOptionPane
				.showOptionDialog(
						this,
						"<html><div align='center'><h3>Welche Variante m\u00F6chten Sie spielen?</h3></div>"
								+ "<ul><li><u>Klassisch</u>"
								+ " - alle Steine werden versteckt dargestellt.<br>"
								+ "Die Spieler decken wechselweise zwei Steine auf und merken sich<br>"
								+ " deren Position.<br>"
								+ "Angeklickte Stein-Paare werden für zwei Sekunden gezeigt.<br>"
								+ "Zeigen sie identische Muster werden sie gel\u00F6scht.<br>"
								+ " Wenn sie keine identischen Muster zeigen, werden sie wieder<br>umgedreht."
								+ "</li>"
								+ "<li><u>Offen</u> - alle Steine werden zun\u00E4chst offen gezeigt.<br>"
								+ "Die Spieler pr\u00E4gen sich die Positionen der Steine ein und <br>"
								+ "drehen alle anschlie\u00DFend herum um sie paarweise zu erraten.</li></ul></html>",
						"Memory - Modus w\u00E4hlen",
						JOptionPane.YES_NO_CANCEL_OPTION,
						JOptionPane.PLAIN_MESSAGE, ico, options, options[0]);
	}

	/**
	 * GUI anlegen
	 */
	private void initGUI() {
		// Button
		turnButt = new JButton("Umdrehen");
		if (mode == 0)
			turnButt.setEnabled(false);
		turnButt.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!start) {
					// Zeitmessung starten
					start = true;
					userPanels[activeUser].runTime(true);
				}
				switchColors();
			}
		});
		closeButt = new JButton("Schlie\u00DFen");
		closeButt.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});
		newButt = new JButton("Neu");
		newButt.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Panel leeren
				MemoryGUI.this.gridPanel.removeAll();

				if (!initLayout())
					return;
				steinSize = gridPanel.getWidth()/gridCount;

				createMemorySteinArray();
				// klassische Version
				if (mode == 0) {
					// Rueckseite zeigen
					switchColors();
					// Memory zunaechst deaktivieren bis start gedrueckt wird
					enableMemory(false);
					// Start-Button aktivieren
					startButt.setEnabled(true);
				}

				showRest();
				// alle User-Labels und die Zeitmessung zuruecksetzen
				for (UserPanel up : userPanels) {
					up.reset();
					up.resetTime();
				}
				MemoryGUI.this.setLocationRelativeTo(null);
				MemoryGUI.this.repaint();
				MemoryGUI.this.validate();
			}
		});
		startButt = new JButton("Start");
		if (mode == 1)
			startButt.setEnabled(false);
		startButt.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				enableMemory(true);
				MemoryGUI.this.userPanels[activeUser].runTime(true);
				startButt.setEnabled(false);
			}
		});
		// Buttons unten
		buttonPanel = new JPanel(new FlowLayout());
		buttonPanel.add(turnButt);
		buttonPanel.add(startButt);
		buttonPanel.add(closeButt);
		buttonPanel.add(newButt);
		countField = new JTextField(new Integer(gridCount).toString(), 2);
		buttonPanel.add(countField);
		// Label oben
		restLabel = new JLabel(" ");
		// Spielflaeche
		gridPanel = new JPanel(new GridLayout(gridCount, gridCount));

		// alles zusammenschrauben
		this.add(userPanels[0], BorderLayout.WEST);
		this.add(userPanels[1], BorderLayout.EAST);
		this.add(restLabel, BorderLayout.NORTH);
		this.add(gridPanel, BorderLayout.CENTER);
		this.add(buttonPanel, BorderLayout.SOUTH);

		// Layout anlegen
		initLayout();
	} // Ende initGUI

	/**
	 * Einrichten des Layouts. Es muss nach dem Wechsel der Steinanzahl neu
	 * geladen werden.
	 * 
	 * @return boolean, <code>true</code> bei Erfolg
	 */
	private boolean initLayout() {
		// nur falls Zahl im Textfeld, Layout neu organisieren
		if (!countField.getText().matches("\\d+")) {
			return false;
		}
		gridCount = new Integer(countField.getText()).intValue();
		if (gridCount > 30) {
			JOptionPane.showMessageDialog(this,
					"Mehr als 30 Steine/Reihe sind nicht zul\u00E4ssig!");
			return false;
		}
		// nur ganze Zahlen akzeptieren, ggf. aufrunden und auf's
		// Textfeld setzen
		if (gridCount % 2 != 0) {
			++gridCount;
			countField.setText(new Integer(gridCount).toString());
		}
		// Panelgroesse muss explizit eingestellt werden, um die Dimensionen
		// beim Vergroessern zu wahren
		gridPanel.setLayout(new GridLayout(gridCount, gridCount));
		return true;
	}

	/**
	 * 
	 * @return MemoryStein, der zuletzt angeklickte Spielstein
	 */
	public MemoryStein getLastStein() {
		return lastStein;
	}

	/**
	 * Registriert den zuletzt angeklickten Spielstein
	 * 
	 * @param lastSt
	 *            MemoryStein, der zuletzt angeklickte Spielstein
	 */
	public void setLastStein(MemoryStein lastSt) {
		this.lastStein = lastSt;
	}

	/**
	 * Initialisiert die Spielsteine
	 */
	private void createMemorySteinArray() {
		steine = new MusterStein[gridCount * gridCount];
		// Steinfarben
		Color color = createColor();
		// alle Steine erstellen und faerben
		// Zufallskomponente zur Gestaltung der Grafik
		int zufall = createZufall();
		Polygon[] poly = { new Cross(steinSize), new LinksStreifen(steinSize),
				new RechtsStreifen(steinSize), new DreieckLinksOben(steinSize),
				new DreieckRechtsOben(steinSize),
				new DreieckRechtsUnten(steinSize),
				new DreieckLinksUnten(steinSize), new RechteckLinks(steinSize),
				new RechteckRechts(steinSize), new RechteckOben(steinSize),
				new RechteckUnten(steinSize), new Rechteck(steinSize),
				new Raute(steinSize), new DreieckOben(steinSize),
				new DreieckUnten(steinSize), new DreieckLinks(steinSize),
				new DreieckRechts(steinSize)};
		long ident = System.nanoTime();
		for (int i = 0; i < gridCount * gridCount; i++) {
			// fuer jeden zweiten Stein eine neue Farbe und einen neuen
			// Zufallswert zur Dimensionierung der Grafik erstellen
			if (i % 2 == 0) {
				ident = System.nanoTime();
				color = createColor();
				zufall = createZufall();
			}
			// Stein erzeugen und ins Array einfuegen
			steine[i] = new MusterStein(ident, color, this, steinSize, zufall,
					//poly[zufall / (steinSize / 15)]);
					poly[zufall]);
		}
		// Steine mischen
		steine = arrayShuffle(steine);
		// Steine auf die Spielflaeche setzen
		for (int i = 0; i < steine.length; i++) {
			gridPanel.add(steine[i]);
		}
	}

	/**
	 * Mischt das Stein-Array zuf&auml;llig
	 * 
	 * @param stein
	 *            Array, das zu mischende MusterStein-Array
	 * @return das gemischte MusterStein-Array
	 */
	private MusterStein[] arrayShuffle(MusterStein[] stein) {
		MusterStein tmp;
		int rand;
		Random r = new Random();
		for (int i = 0; i < stein.length; i++) {
			rand = r.nextInt(stein.length);
			tmp = stein[i];
			stein[i] = stein[rand];
			stein[rand] = tmp;
		}
		return stein;
	}

	/**
	 * 'Dreht' alle Steine von der Vorder- auf die R&uuml;ckseite und umgekehrt
	 * 
	 * @param stein
	 *            Array der Steine
	 */
	private void switchColors() {
		for (int i = 0; i < steine.length; i++) {
			// wenn Vorderseite gezeigt wird Rueckseite zeigen und umgekehrt
			steine[i].setFrontVisible(!steine[i].isFrontVisible());
			steine[i].repaint();
		}
	}

	/**
	 * Zufall erzeugen, dessen Wert begrenzt ist. Die Methode erzeugt einen
	 * Zufallswert und testet diesen rekursiv gegen eine von der Steingroesse
	 * und einer Konstanten abhaengigen Grenze.
	 */
	private int createZufall() {
		Random rand = new Random();
		// brauchbare Groesse (0-16) sicherstellen, da die Groesse des
		// Polygon-Arrays bei 17 liegt
		int zufall = rand.nextInt(17);
			return zufall;
	}

	/**
	 * Erstellt eine Zufallsfarbe
	 * 
	 * @return die erstellte Zufallsfarbe
	 */
	private Color createColor() {
		Random rand = new Random();
		int r = rand.nextInt(255);
		int g = rand.nextInt(255);
		int b = rand.nextInt(255);
		Color c = new Color(r, g, b);
		return c;
	}

	/**
	 * Stellt die Anzahl der noch zu erratenden Spielstein-Paare im Label oben
	 * dar. Wenn dieser Wert 0 ist, wird eine Gl&uuml;ckwunschmeldung mit der
	 * Nennung der Spielernummer erzeugt.
	 */
	public void showRest() {
		int count = 0;
		for (int i = 0; i < steine.length; i++) {
			if (!steine[i].isVisible())
				count++;
		}
		int i = (steine.length - count) / 2;
		restLabel.setText("Rest: " + new Integer(i).toString() + "/"
				+ new Integer(steine.length / 2).toString() + " Paare");

		if (i == 0) {
			int sieger;
			if (userPanels[0].getTreffer() == userPanels[1].getTreffer())
				sieger = userPanels[0].getTime() < userPanels[1].getTime() ? 1
						: 2;
			else
				sieger = userPanels[0].getTreffer() > userPanels[1]
						.getTreffer() ? 1 : 2;
			userPanels[0].runTime(false);
			userPanels[1].runTime(false);
			JOptionPane.showMessageDialog(
					this,
					"Herzlichen Gl\u00FCckwunsch!\nSieger ist Spieler "
							+ sieger + " mit "
							+ userPanels[sieger - 1].getTreffer()
							+ " Treffern in "
							+ userPanels[sieger - 1].printTime() + " Minuten.");
		}
	}

	/**
	 * Incrementiert die Summe der Treffer des aktuellen Spielers
	 */
	public void setSuccess() {
		userPanels[activeUser]
				.setTreffer(userPanels[activeUser].getTreffer() + 1);
	}

	/**
	 * &Auml;ndert den Benutzer auf den jeweils anderen und setzt erst alle
	 * Checkboxen auf <code>false</code> und dann die des aktuellen Nutzers auf
	 * <code>true</code>
	 */
	public void switchUser() {
		userPanels[activeUser].stopTime();
		activeUser = activeUser == 0 ? 1 : 0;
		for (UserPanel up : userPanels) {
			up.checkUserCB(false);
		}
		userPanels[activeUser].checkUserCB(true);
		userPanels[activeUser].runTime(true);
	}

	public int getMode() {
		return mode;
	}

	/**
	 * (de)aktiviert alle Spielsteine
	 * 
	 * @param enable
	 *            , boolean <code>true</code> aktiviert alle Spielsteine,
	 *            <code>false</code> deaktiviert sie
	 */
	public void enableMemory(boolean enable) {
		for (MemoryStein st : steine) {
			st.setEnabled(enable);
		}
	}
}
