import java.awt.Color;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;

/**
 * @MemoryStein.java     14.03.2011
 * build: 14.03.2011
 * Copyright 2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Memory
 * Memory is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Memory is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Memory;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */

/**
 * Elternklasse der Spielsteine des Memory-Spiels
 * 
 */
public class MemoryStein extends JButton implements ActionListener {

	/**
	 * Hilfsvariable, steuert die Sichtbarkeit der Steine. Erratene Steine
	 * werden unsichtbar.
	 */
	private boolean isFrontVisible = true;
	protected final Color backColor = new Color(255, 200, 20);
	protected MemoryGUI parent;
	protected long ident;

	public MemoryStein(long ident, MemoryGUI parent) {
		this.parent = parent;
		this.ident = ident;
		this.addActionListener(this);
		this.setOpaque(true);
		this.setBackground(backColor);
	}

	/**
	 * 
	 * @return long, die Identifikationsnummer eines 'Spielsteins'
	 */
	public long getIdent() {
		return ident;
	}

	/**
	 * 
	 * @return Color, die Farbe der 'R&uuml;ckseite'
	 */
	public Color getBackColor() {
		return backColor;
	}

	/**
	 * Steuert die Sichtbarkeit von 'Vorder- und R&uuml;ckseite eines
	 * Spielsteins'.
	 * 
	 * @param isFrontVisible
	 *            boolean, <code>true</code> wenn die 'Voderseite' eines
	 *            Spielsteins sichtbar sein soll, sonst <code>false</code>
	 */
	protected void setFrontVisible(boolean isFrontVisible) {
		this.isFrontVisible = isFrontVisible;
	}

	/**
	 * Gibt die Sichtbarkeit der Vorderseite zur&uuml;ck. <code>true</code wenn
	 * diese sichtbar ist.
	 * 
	 * @return
	 */
	protected boolean isFrontVisible() {
		return isFrontVisible;
	}

	/**
	 * Steuert das gesamte Handling des Ratens durch die Erkennung der
	 * Stein-&Uuml;bereinstimmungen
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		// offener Modus
		if (parent.getMode() == 1) {
			// wenn auf den gleichen Stein geklickt wird
			if (parent.getLastStein() == this) {
				parent.setLastStein(this);
				Toolkit.getDefaultToolkit().beep();
				parent.switchUser();
				// wenn auf einen anderen Stein geklickt wird und kein
				// Vorgaenger
				// existiert
			} else if (parent.getLastStein() != this
					&& parent.getLastStein() == null) {
				parent.setLastStein(this);
				// wenn auf einen anderen Stein geklickt wird und ein Vorgaenger
				// existiert
			} else if (parent.getLastStein() != this
					&& parent.getLastStein() != null) {
				// wenn Farben identisch
				if (ident == parent.getLastStein().getIdent()) {
					// Steine deaktivieren
					parent.getLastStein().setEnabled(false);
					parent.getLastStein().setVisible(false);
					this.setEnabled(false);
					this.setVisible(false);
					parent.setLastStein(null);
					parent.setSuccess();
					parent.showRest();
					// wenn Farben nicht identisch
				} else if (ident != parent.getLastStein().getIdent()) {
					this.setBackground(backColor);
					parent.getLastStein().setBackground(backColor);
					parent.setLastStein(null);
					Toolkit.getDefaultToolkit().beep();
					parent.switchUser();
				}
			}
		} else if (parent.getMode() == 0) {
			// wenn beide Steine sichtbar sind und auf einen der beiden geklickt
			// wird: Abbruch
			if (this.isFrontVisible
					&& parent.getLastStein().isFrontVisible
					&& (e.getSource() == this || e.getSource() == parent
							.getLastStein())) {
				Toolkit.getDefaultToolkit().beep();
				return;
			}

			// wenn auf den Vorgaenger-Stein geklickt wird
			if (parent.getLastStein() == this) {
				parent.setLastStein(this);
				Toolkit.getDefaultToolkit().beep();
				parent.switchUser();
				// wenn auf einen Stein geklickt wird und kein
				// Vorgaenger
				// existiert
			} else if (parent.getLastStein() != this
					&& parent.getLastStein() == null) {
				parent.setLastStein(this);
				this.setFrontVisible(true);
				// wenn auf einen anderen Stein als den Vorgaenger-Stein
				// geklickt wird und ein Vorgaenger existiert
			} else if (parent.getLastStein() != this
					&& parent.getLastStein() != null) {
				// Stein sichtbar setzen
				this.setFrontVisible(true);
				// wenn Farben identisch beide verschwinden lassen
				if (ident == parent.getLastStein().getIdent()) {
					// Steine deaktivieren
					parent.getLastStein().setEnabled(false);
					parent.getLastStein().setVisible(false);
					this.setEnabled(false);
					this.setVisible(false);
					parent.setLastStein(null);
					parent.setSuccess();
					parent.showRest();
					// wenn Farben nicht identisch beide fuer eine Zeit zeigen
					// und dann wieder umdrehen
				} else if (ident != parent.getLastStein().getIdent()) {
					this.setFrontVisible(true);
					parent.getLastStein().setFrontVisible(true);
					Thread t = new Thread(new Runnable() {
						public void run() {
							try {
								parent.enableMemory(false);
								Thread.sleep(2000);
								parent.enableMemory(true);
							} catch (InterruptedException e) {
							}
							MemoryStein.this.setFrontVisible(false);
							if (parent.getLastStein() != null) {
								parent.getLastStein().setFrontVisible(false);
								parent.getLastStein().repaint();
							}
							MemoryStein.this.repaint();
							parent.setLastStein(null);
						}
					});
					parent.switchUser();
					t.start();
				}
			}
		}
	}
}
