import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.RenderingHints;

/**
 * @MusterStein.java     15.03.2011
 * build: 15.03.2011
 * Copyright 2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Memory
 * Memory is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Memory is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Memory;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */

/**
 * Die Klasse realisiert einen 'Spielstein' des Memory-Spiels.
 * 
 */
public class MusterStein extends MemoryStein {

	/**
	 * Grundfarbe Vorderseite
	 */
	private Color frontColor = Color.WHITE;
	/**
	 * Zeichnungsfarben
	 */
	private Color col;
	/**
	 * Position der Zeichnung
	 */
	int x, y;
	/**
	 * Steingroesse[px], dient zur Dimensionierung der Zeichnung.
	 * 
	 * @see:MemoryGUI.steinSize
	 */
	int size;
	/**
	 * Zufallswert zur Steuerung der Zeichnungsgr&ouml;sse und -position, nimmt
	 * Werte zwischen 0 und size*0.6 an
	 */
	int zufall;
	/**
	 * Identifiziert die zwei Steine eines Seinpaares, Bei identischen Steinen
	 * besitzen diese den gleichen Wert.
	 */
	long ident;
	/**
	 * Zeichnung auf der 'Vorderseite' eines 'Steins'
	 */
	Polygon poly;

	/**
	 * @param ident
	 *            Identifizierungs-Index zur Verifizierung der Stein-Paare
	 * @param col
	 *            Zeichnungsfarbe 2
	 * @param parent
	 *            MemoryGUI, Referenz auf das Hauptfenster (Controller)
	 * @param size
	 *            Steingr&ouml;sse
	 * @param zufall
	 *            Zufallswert zur Steuerung der Zeichnung
	 */
	public MusterStein(long ident, Color col, MemoryGUI parent, int size,
			int zufall, Polygon poly) {
		super(ident, parent);
		this.col = col;
		this.size = size;
		this.zufall = zufall;
		this.poly = poly;
	} // Konstruktor

	/**
	 * Realisiert die Zeichnung auf dem Spielstein. Farbigkeit und Formen werden
	 * durch einen Zufallswert gesteuert. Das Muster selbst wird bei der
	 * Instanzierung des Spielsteins bestimmt. Das Umschalten von Vorder- auf
	 * R&uuml;ckseite erfolgt &uuml;ber die Hilfsvariable
	 * <code>isFrontVisible</code>.
	 */
	public void paintComponent(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		// Vorderseite
		if (isFrontVisible()) {
			// Grundflaeche
			g2d.setColor(frontColor);
			g2d.fillRect(0, 0, size, size);

			// Polygon
			g2d.setColor(col);
			g2d.fillPolygon(poly);

		} else { // Rueckseite
			g.setColor(backColor);
			g.fillRect(0, 0, size, size);
		}
	}
}

class Cross extends Polygon {
	public Cross(int size) {
		super(new int[] { 0, size/6, size / 2, size - size/6, size, size, size / 2 + 2,
				size, size, size - size/6, size / 2, size/6, 0, 0, size / 2 - 2, 0 },
				new int[] { 0, 0, size / 2 - 2, 0, 0, size/6, size / 2, size - size/6,
						size, size, size / 2 + 2, size, size, size - size/6,
						size / 2, size/6 }, 16);
	}
}

class LinksStreifen extends Polygon {
	public LinksStreifen(int size) {
		super(new int[] { 0, 0, size/5, size, size, size - size/5 }, new int[] { size/5, 0, 0,
				size - size/5, size, size }, 6);
	}
}

class RechtsStreifen extends Polygon {
	public RechtsStreifen(int size) {
		super(new int[] { size - size/5, size, size, size/5, 0, 0 }, new int[] { 0, 0, size/5,
				size, size, size - size/5 }, 6);
	}
}

class DreieckLinksOben extends Polygon {
	public DreieckLinksOben(int size) {
		super(new int[] { 0, size, 0 }, new int[] { 0, 0, size }, 3);
	}
}

class DreieckLinksUnten extends Polygon {
	public DreieckLinksUnten(int size) {
		super(new int[] { 0, size, 0 }, new int[] { 0, size, size }, 3);
	}
}

class DreieckRechtsOben extends Polygon {
	public DreieckRechtsOben(int size) {
		super(new int[] { 0, size, size }, new int[] { 0, 0, size }, 3);
	}
}

class DreieckRechtsUnten extends Polygon {
	public DreieckRechtsUnten(int size) {
		super(new int[] { size, size, 0 }, new int[] { 0, size, size }, 3);
	}
}

class RechteckUnten extends Polygon {
	public RechteckUnten(int size) {
		super(new int[] { 0, size, size, 0 }, new int[] { size / 2, size / 2,
				size, size }, 4);
	}
}

class RechteckOben extends Polygon {
	public RechteckOben(int size) {
		super(new int[] { 0, size, size, 0 }, new int[] { 0, 0, size / 2,
				size / 2 }, 4);
	}
}

class RechteckLinks extends Polygon {
	public RechteckLinks(int size) {
		super(new int[] { 0, size / 2, size / 2, 0 }, new int[] { 0, 0, size,
				size }, 4);
	}
}

class RechteckRechts extends Polygon {
	public RechteckRechts(int size) {
		super(new int[] { size / 2, size, size, size / 2 }, new int[] { 0, 0,
				size, size }, 4);
	}
}

class DreieckOben extends Polygon{
	public DreieckOben(int size){
		super(new int[] { 0, size, size / 2 }, new int[] { 0, 0,
				size }, 3);
	}
}

class DreieckUnten extends Polygon{
	public DreieckUnten(int size){
		super(new int[] { size / 2, size, 0 }, new int[] { 0, size,
				size }, 3);
	}
}

class DreieckLinks extends Polygon{
	public DreieckLinks(int size){
		super(new int[] { 0, size, 0 }, new int[] { 0, size/2,
				size }, 3);
	}
}

class DreieckRechts extends Polygon{
	public DreieckRechts(int size){
		super(new int[] { 0, size, size }, new int[] { size/2, 0,
				size }, 3);
	}
}

class Rechteck extends Polygon {
	public Rechteck(int size){
		super(new int[] {0, size, size, 0}, new int[] {0, 0, size, size}, 4);
	}
}

class Raute extends Polygon {
	public Raute(int size){
		super(new int[] {0, size/2, size, size/2}, new int[] {size/2, 0, size/2, size}, 4);
	}
}
