import java.awt.GridLayout;
import java.text.DecimalFormat;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * @UserPanel.java     20.03.2011
 * build: 20.03.2011
 * Copyright 2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Memory
 * Memory is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Memory is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Memory;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */

/**
 * Realisiert ein Panel zur Anzeige des akriven Spielers und seiner korrekt
 * erkannten Steinpaare
 * 
 */
public class UserPanel extends JPanel {

	/**
	 * Die Checkbox markiert den gerade aktiven Spieler. Die Aktivierung findet
	 * nach einem Fehler des Gegenspielers statt. Als Fehler gilt das Anklicken
	 * eines nicht-identischen zweiten Spielsteins oder das wiederholte
	 * Anklicken eines Spielsteins
	 */
	private JCheckBox userCB;
	/**
	 * Das Label zur Anzeige der gefundenen korrekten Spielsteinpaare des
	 * Spielers
	 */
	private JLabel trefferLabel;

	/**
	 * Label zur Anzeige der ben&ouml;tigten Zeit
	 */
	private JLabel timeLabel;

	private JPanel mainPanel;
	/**
	 * Die Anzahl der korrekt erkannten Spielsteinpaare
	 */
	private int treffer;
	/**
	 * ben&oml;tigte Zeit
	 */
	private int time = 0;
	/**
	 * Hilfsvariable zur Steuerung der Zeitmessung <code>true</code> = Zeit
	 * l&auml;uft
	 */
	private boolean run = false;

	public UserPanel(boolean selected, int user) {
		userCB = new JCheckBox("Spieler " + user + " aktiv", selected);
		userCB.setEnabled(false);
		trefferLabel = new JLabel("Treffer: " + treffer);
		timeLabel = new JLabel("Zeit: 00:00");
		mainPanel = new JPanel(new GridLayout(3, 1));
		mainPanel.add(userCB);
		mainPanel.add(trefferLabel);
		mainPanel.add(timeLabel);
		this.add(mainPanel);
	}

	/**
	 * (de)aktiviert die Checkbox zur Anzeige des gerade aktiven Spielers
	 * 
	 * @param selected
	 *            boolean, zu setzender Aktivit&auml;tszustand der Checkbox
	 */
	public void checkUserCB(boolean selected) {
		userCB.setSelected(selected);
	}

	/**
	 * Setzt das Anzeigelabel mit den Treffern des Spielers auf 0.
	 */
	public void reset() {
		trefferLabel.setText("Treffer: 0");
	}

	/**
	 * Setzt die Zahl der korrekt erkannten Spielstein-Paare auf das Label
	 * 
	 * @param treffer
	 *            int, Anzahl der korrekt erkannten Spielstein-Paare
	 */
	public void setTreffer(int treffer) {
		this.treffer = treffer;
		trefferLabel.setText("Treffer: " + this.treffer);
	}

	/**
	 * Gibt die Anzahl der korrekt erkannten Spielstein-Paare zur&uuml;ck
	 * 
	 * @return int, die Anzahl der korrekt erkannten Spielstein-Paare
	 */
	public int getTreffer() {
		return treffer;
	}

	/**
	 * liefert die vom Nutzer ben&ouml;tigte Gesamtzeit
	 * 
	 * @return int, die ben&ouml;tigte Zeit in Sekunden
	 */
	public int getTime() {
		return time;
	}

	/**
	 * Liefert die vom Benutzer ben&ouml;tigte Zeit im String-Format
	 * 
	 * @return String im Format mm:ss
	 */
	public String printTime() {
		int minutes = this.time / 60;
		int seconds = this.time % 60;
		DecimalFormat format = new DecimalFormat("00");
		String time = format.format(new Integer(minutes)) + ":"
				+ format.format(new Integer(seconds));
		timeLabel.setText("Zeit: " + time);
		return time;
	}

	/**
	 * Setzt die Hilfsvariable <code>run</code auf <code>false</code> und stopt
	 * so den Thread zur Zeitmessung. Gleichzeitig wird die Zeit auf dem Panel
	 * dargestellt.
	 */
	public void stopTime() {
		run = false;
		printTime();
	}

	/**
	 * Managed die Zeitmessung
	 * 
	 * @param run
	 *            boolean <code>true</code> startet die Zeitmessung, <code>false</code> stopt sie.
	 */
	void runTime(boolean run) {
		this.run = run;
		Thread t = new Thread(new Runnable() {
			public void run() {
				while (UserPanel.this.run) {
					++time;
					try {
						Thread.sleep(1000);
						printTime();
					} catch (InterruptedException e) {
					}
				}
			}
		});
		t.start();
	}

	/**
	 * Setzt die Zeitmessung zur&uuml;ck.
	 */
	public void resetTime() {
		time = 0;
		run = false;
		printTime();
	}
}
